# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import logging
import xml.etree.cElementTree as ET
from collections import OrderedDict

from driverng import MessageV2, utils, Worklist

logger = logging.getLogger("driverng.adagioxml")


class Driver(MessageV2):
    SPEC = {"code": "adagioxml", "label": "Adagio XML"}
    ENCODING = "utf8"

    OPTIONS = [
        MessageV2.Option(
            "atb_order_classtcode",
            label="Ordonner les antibiotiques selon la classe de texte codé",
        ),
        MessageV2.BooleanOption(
            "adagio_integration_bmr", label="Intégration des BMR", default=False
        ),
        MessageV2.Option(
            "adagio_default_cmi_code",
            label="Code CMI par défaut en cas d'absence de CMI",
            default="",
        ),
        MessageV2.BooleanOption(
            "adagio_cmi_measured",
            label="Traiter la CMI mesurée au lieu de la CMI estimée (balise measured_MIC)",
            default=False,
        ),
        MessageV2.Option(
            "adagio_hospitalization_code",
            label="Numéro d'hospitalisation par défaut (balise hospitalization si absence de numéro d'hospitalisation",
            default="",
        ),
        MessageV2.Option(
            "adagio_prefixe_germe",
            label="Préfixe des textes codés des germes (par défaut G)",
            default="G",
        ),
        MessageV2.Option(
            "adagio_prefixe_antibio",
            label="Préfixe des textes codés des antibiotiques (par défaut A)",
            default="A",
        ),
        MessageV2.BooleanOption(
            "adagio_get_diametre",
            label="L'adagio renvoie le diamètre d'inhibition dans les ATB.",
            default=False,
        ),
    ]

    def __init__(self, **kwargs):
        self.atb_order_classtcode = kwargs.pop("atb_order_classtcode", None)
        self.adagio_integration_bmr = kwargs.pop("adagio_integration_bmr", False)
        self.adagio_default_cmi_code = kwargs.pop("adagio_default_cmi_code", "")
        self.adagio_cmi_measured = kwargs.pop("adagio_cmi_measured", False)
        self.adagio_hospitalization_code = kwargs.pop("adagio_hospitalization_code", "")
        self.adagio_prefixe_germe = kwargs.pop("adagio_prefixe_germe", "G")
        self.adagio_prefixe_antibio = kwargs.pop("adagio_prefixe_antibio", "A")
        self.adagio_get_diametre = kwargs.pop("adagio_get_diametre", False)
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        return ET.fromstring(data.encode("utf8"))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, "utf-8"))

    def encode_worklist(self, worklist, context):
        message = ET.Element("data")
        patients = ET.SubElement(message, "patients")
        empty = True
        for sample, _ in worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            seq = 1
            patient = ET.SubElement(patients, "patient")
            patient.set("id", "1")
            ET.SubElement(patient, "patient_id").text = sample.lab_patient_id
            name = ET.SubElement(patient, "name")
            ET.SubElement(name, "lastname").text = sample.last_name
            ET.SubElement(name, "firstname").text = sample.first_name
            if sample.birthdate:
                ET.SubElement(patient, "birthdate").text = utils.format_date(
                    sample.birthdate, fmt="%Y%m%d"
                )
            if sample.sex == "F":
                ET.SubElement(patient, "sex").text = "1"
            elif sample.sex == "M":
                ET.SubElement(patient, "sex").text = "2"
            samples = ET.SubElement(patient, "samples")
            tube_sample = ET.SubElement(samples, "sample")
            tube_sample.set("id", str(seq))
            for order in sample.orders:
                ET.SubElement(tube_sample, "sample_code").text = sample.sample_id
                ET.SubElement(tube_sample, "sample_type_code").text = order.code
                ET.SubElement(tube_sample, "exam_date").text = utils.format_date(
                    sample.request_date, fmt="%Y%m%d"
                )
                empty = False
            sample_prescriber = ET.SubElement(tube_sample, "sample_prescriber")
            physician = [p for p in sample.physicians if p.type.lower() == "prescriber"]
            ET.SubElement(sample_prescriber, "sample_prescriber_code").text = (
                physician[0].code if physician else ""
            )
            ET.SubElement(sample_prescriber, "sample_prescriber_lastname").text = (
                physician[0].last_name if physician else ""
            )

            # Envoi du code du service transmetteur
            if sample.service or sample.location:
                code_service = ""
                if sample.service:
                    code_service = sample.service
                elif sample.location:
                    code_service = sample.location
                hospitalizations = ET.SubElement(patient, "hospitalizations")
                hospitalization = ET.SubElement(hospitalizations, "hospitalization")
                hospitalization.set("id", "1")
                ET.SubElement(hospitalization, "hospitalization_code").text = (
                    sample.alternate_patient_id
                    if sample.alternate_patient_id
                    else self.adagio_hospitalization_code
                )
                hospitalization_date = ET.SubElement(
                    hospitalization, "hospitalization_date"
                )
                ET.SubElement(
                    hospitalization_date, "start_hospitalization_date"
                ).text = utils.format_date(sample.request_date, fmt="%Y%m%d")
                wards = ET.SubElement(hospitalization, "wards")
                ward = ET.SubElement(wards, "ward")
                ward.set("id", "1")
                ET.SubElement(ward, "ward_code").text = code_service

            examens = ET.SubElement(tube_sample, "analysiss")
            for result in sample.results:
                id_germe = ""
                rank_germ = ""
                if result.resu_json and "germes" in result.resu_json:
                    for index, resu_json_germe in enumerate(result.resu_json.germes, 1):
                        try:
                            key_germe = [
                                cle
                                for cle in resu_json_germe.keys()
                                if cle.lower() == "id"
                            ]
                            id_germe = resu_json_germe.get(
                                key_germe[0] if key_germe else None
                            )
                            if isinstance(id_germe, list):
                                id_germe = ""
                        except KeyError:
                            id_germe = ""

                        try:
                            rank_germ = resu_json_germe.get("rang")
                        except KeyError:
                            rank_germ = ""
                        if id_germe:
                            examen = ET.SubElement(examens, "analysis")
                            examen.set("id", str(index))
                            germ = ET.SubElement(examen, "germ")
                            if id_germe.startswith("G_"):
                                ET.SubElement(germ, "germ_code").text = id_germe[2:]
                            else:
                                ET.SubElement(germ, "germ_code").text = id_germe
                        if id_germe and rank_germ:
                            ET.SubElement(germ, "germ_rank").text = str(rank_germ)
                            id_germe = ""
                            rank_germ = ""
                if result.value.startswith("G_") or id_germe:
                    if result.value.startswith("G_"):
                        examen = ET.SubElement(examens, "analysis")
                        germ = ET.SubElement(examen, "germ")
                        ET.SubElement(germ, "germ_code").text = result.value[2:]

            seq += 1
        if not empty:
            return message

    def decode_message(self, message):
        if message.findall("query"):
            worklist = Worklist(context="query")
            for query in message.findall("query"):
                for sample in query.findall("sample"):
                    worklist.get_or_create(
                        type="patient", sample_id=query.findtext("sample")
                    )
            return worklist

        worklist = Worklist(context="result")

        if not message.find("patients"):
            return {}
        samples_results = OrderedDict()
        patients = message.find("patients")
        for patient in patients.findall("patient"):
            samples = patient.find("samples")
            for sample in samples.findall("sample"):
                sample_id = sample.findtext("sample_code")
                sample_results = samples_results.setdefault(sample_id, OrderedDict())
                test = sample.findtext("sample_type_code")
                germe = "{prefix}_{germ_code}".format(
                    prefix=self.adagio_prefixe_germe,
                    germ_code=sample.findtext(
                        "./analysiss/analysis/germ/germ_code"
                    ).upper(),
                )
                result = {
                    "code": test,
                    "comments": [],
                    "date": datetime.datetime.strptime(
                        sample.findtext("./analysiss/analysis/exam_validation_date"),
                        "%Y%m%d",
                    ),
                    "value": [
                        {
                            "antibiogram": [],
                            "seq": sample.findtext(
                                "./analysiss/analysis/germ/germ_rank"
                            ),
                            "value": germe,
                        }
                    ],
                }
                antibiotiques = OrderedDict()
                resistance_markers = []
                for antibiotic in sample.findall("./analysiss/analysis/results/result"):
                    code_bmr = ""
                    phenotype_code = antibiotic.findtext("phenotype_code")
                    if self.adagio_integration_bmr and phenotype_code:
                        code_bmr = "BMR_{phenotype_code}".format(
                            phenotype_code=phenotype_code
                        )
                        code = code_bmr
                    elif antibiotic.findtext("antibiotic_code"):
                        code = "{prefix}_{antibiotic_code}".format(
                            prefix=self.adagio_prefixe_antibio,
                            antibiotic_code=antibiotic.findtext("antibiotic_code"),
                        )
                    elif not phenotype_code:
                        logger.warning(
                            "On ignore result (id {}) sans antibiotic_code".format(
                                antibiotic.get("id")
                            )
                        )
                        continue
                    # Ignorer les résultats des antibiotiques sans
                    # interprétation et sans valeur estimée de CMI
                    # sauf pour les BMR et lorsque l'option pour
                    # prendre en compte la cmi mesurée est activée.
                    # On n'ignore cependant pas les résultats s'ils
                    # ont un phénotype
                    if (
                        not code_bmr
                        and not self.adagio_cmi_measured
                        and not antibiotic.findtext("estimated_MIC")
                        and not antibiotic.findtext("final_RIS")
                        and not phenotype_code
                    ):
                        if self.adagio_get_diametre:
                            if not antibiotic.findtext("diameter"):
                                continue
                        else:
                            continue
                    default_atb = {"antibiotic": code, "sensitivity": None, "mic": None}
                    if self.adagio_get_diametre:
                        default_atb["diameter"] = None
                    antibio = antibiotiques.setdefault(code.upper(), default_atb)
                    if phenotype_code:
                        if self.adagio_integration_bmr:
                            antibio["antibiotic"] = code_bmr
                            antibio["sensitivity"] = "BMR"
                            antibio["mic"] = "#"
                        else:
                            resistance_markers.append(phenotype_code)
                    if self.adagio_default_cmi_code:
                        if (
                            self.adagio_cmi_measured
                            and not antibiotic.findtext("measured_MIC")
                            or not antibiotic.findtext("estimated_MIC")
                        ):
                            antibio["mic"] = self.adagio_default_cmi_code
                    elif not self.adagio_cmi_measured and antibiotic.findtext(
                        "estimated_MIC"
                    ):
                        antibio["mic"] = antibiotic.findtext("estimated_MIC")
                    elif self.adagio_cmi_measured and antibiotic.findtext(
                        "measured_MIC"
                    ):
                        antibio["mic"] = antibiotic.findtext("measured_MIC")
                    if antibiotic.findtext("final_RIS"):
                        antibio["sensitivity"] = antibiotic.findtext("final_RIS")
                    if (
                        self.adagio_get_diametre
                        and antibiotic.findtext("antibiotic_code")
                        and antibiotic.findtext("diameter")
                    ):
                        antibio["diameter"] = antibiotic.findtext("diameter")
                    if antibiotiques:
                        result.get("value")[0]["antibiogram"] = [
                            dict(sorted(atb.items()))
                            for atb in antibiotiques.values()
                            if (atb.get("sensitivity"), atb.get("mic")) != (None, None)
                        ]
                list_comment_expert = []
                for comment_expert in sample.findall(
                    ".analysiss/analysis/expert_comments/comment"
                ):
                    # Traitement des commentaires experts
                    if (
                        comment_expert.findtext("comment_code") is None
                        or comment_expert.findtext("comment_code") == ""
                    ):
                        continue
                    list_comment_expert.append(comment_expert.findtext("comment_code"))
                # Regrouper les commentaires experts codés
                if list_comment_expert:
                    result["comments"] = list_comment_expert
                    sample_results["COMMENT_CODE"] = {
                        "code": "COMMENT_CODE",
                        "value": list_comment_expert,
                    }
                if resistance_markers:
                    result["phenotypes"] = resistance_markers
                if not result.get("value")[0]["antibiogram"]:
                    logger.info(
                        "Absence de résultat d'antibiogramme pour l'échantillon numéro {}".format(
                            sample_id
                        )
                    )
                sample_results[result["code"]] = result

            for sample_id, data in samples_results.items():
                sample = worklist.get_or_create(sample_id=sample_id, type="patient")
                for key in data:
                    sample.results.add(data.get(key))

        return worklist
