# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng.astm import AstmMessageV2
from driverng import Worklist
from driverng.drivers.alinity import DriverAlinity


class DriverAlinityMh(DriverAlinity):
    SPEC = {"code": "alinity_mh", "label": "Alinity m/h HL7 Biomoléculaire"}

    OPTIONS = [
        AstmMessageV2.Option(
            "alinity_sender_code", label="Code émetteur", default="LISApp"
        ),
        AstmMessageV2.Option(
            "alinity_sender_label", label="Libellé émetteur", default="MainFacility"
        ),
        AstmMessageV2.Option(
            "alinity_receiver_code", label="Code récepteur", default="AlinityApp"
        ),
        AstmMessageV2.Option(
            "alinity_receiver_label", label="Libellé récepteur", default="LabFacility"
        ),
        AstmMessageV2.BooleanOption(
            "alinity_dilution_en_alarme",
            label="Traiter le facteur du dilution comme alarme du résultat",
        ),
        AstmMessageV2.Option(
            "alinity_code_exception",
            label="Code exception à mettre dans le résultat si flag X dans OBX.11",
            default="EXCEPTION",
        ),
    ]

    def __init__(self, **kwargs):
        self.sender_code = kwargs.pop("alinity_sender_code", "LISApp")
        self.sender_label = kwargs.pop("alinity_sender_label", "MainFacility")
        self.receiver_code = kwargs.pop("alinity_receiver_code", "AlinityApp")
        self.receiver_label = kwargs.pop("alinity_receiver_label", "LabFacility")
        self.dilution_en_alarme = kwargs.pop("alinity_dilution_en_alarme", False)
        self.code_exception = kwargs.pop("alinity_code_exception", "EXCEPTION")
        super(DriverAlinityMh, self).__init__(**kwargs)

    def encode_worklist(self, worklist, context):
        message = self.create(
            sending_application=self.sender_code,
            sending_facility=self.sender_label,
            receiving_application=self.receiver_code,
            receiving_facility=self.receiver_label,
            message_date=context["now"],
            control_id=context.get("control_id", int(time.time() * 1000)),
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority", "rerun"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": "CA", worklist.ELSE: "NW"},
        ):
            for sample, action, priority, rerun in patient.samples:
                seq = 1
                for order in sample["orders"]:
                    seg_spm = message.specimens.add(
                        specimen_type=sample.specimen,
                        role_id="P",
                        role_label="Patient",
                        role_coding_system="HL70369",
                        collection_date=sample.collection_date,
                    )
                    seg_sac = seg_spm.infos_samples.add(identifier=sample.sample_id)
                    if not rerun and order.dilution:
                        seg_sac.dilution = order.dilution
                    seg_sac.common_order.add(
                        order_control=action, date_transaction=sample.request_date
                    )
                    if priority == "R":
                        priority_label = "Routine"
                    elif priority == "S":
                        priority_label = "STAT"
                    seg_tq1 = seg_sac.sample_priority_order.add(
                        type_segment="TQ1",
                        priority_code=priority,
                        priority_label=priority_label,
                        priority_coding="HL70485",
                    )
                    seg_obr = seg_tq1.orders.add(
                        placer_order_number=context.get(
                            "placer_order_number", int(time.time() * 1000 + seq)
                        ),
                        test_identifier=order.code,
                        test_text=order.code,
                    )
                    for physician in sample.physicians:
                        seg_obr.ordering_provider = physician.code
                    if rerun and order.dilution:
                        seg_obr.infos_tests.add(
                            universal_service_identifier=order.code,
                            test_label=order.code,
                            test_coding_name="99ABT",
                            auto_dilution_type=order.dilution,
                            dilution_coding="99ABT",
                        )
                    seq += 1
        if len(message.specimens):
            return message
        elif context["query"]:
            for sample_id in [
                seg_qpd.query_name for seg_qpd in context["query"].queries
            ]:
                seg_spm = message.specimens.add(
                    role_id="U", role_label="Unknow", role_coding_system="HL70369"
                )
                seg_sac = seg_spm.infos_samples.add(identifier=sample_id)
                seg_sac.common_order.add(
                    order_control="DC", date_transaction=context["now"]
                )
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_qpd in message.queries:
                sample = worklist.get_or_create(
                    type="patient", sample_id=seg_qpd.query_name
                )
        else:
            worklist = Worklist(context="result")
            if message.specimens:
                for seg_spm in message.specimens:
                    is_qc = seg_spm.role_id == "Q"
                    if is_qc:
                        sample = worklist.get_or_create(type="control")
                    else:
                        sample = worklist.get_or_create(type="patient")
                    for seg_sac in seg_spm.infos_samples:
                        if is_qc:
                            for seg_inv in seg_sac.inventory:
                                sample.sample_id = seg_inv.manufacturer_lot_number
                                sample.control_lot = seg_inv.manufacturer_lot_number
                                sample.control_name = seg_sac.identifier
                        else:
                            sample.sample_id = seg_sac.identifier
                        for seg_obr in seg_sac.orders:
                            if seg_obr.sample_priority:
                                results = seg_obr.sample_priority[0].results
                            elif seg_obr.common_order:
                                results = []
                                if seg_obr.common_order[0].sample_priority:
                                    segs_orc = seg_obr.common_order[0].sample_priority
                                    if segs_orc[0].results:
                                        results = segs_orc[0].results
                            else:
                                results = seg_obr.results
                            for seg_obx in results:
                                code = seg_obx.assay_number
                                if (
                                    "HPV" in seg_obx.result_type_code
                                    and seg_obx.observation_sub_id
                                ):
                                    code = "{}_{}".format(
                                        seg_obx.assay_number, seg_obx.observation_sub_id
                                    )
                                sample.results.add(
                                    code=code,
                                    value=seg_obx.value,
                                    date=seg_obx.date_analysis,
                                    instrument=seg_obx.equipement_identifier[1],
                                    operator=seg_obx.responsible_observer,
                                    unit=seg_obx.units,
                                )
        return worklist
